local addonName, global = ...;

local frame = CreateFrame("Frame", "OSBCP_Easy_Healer_Settings_Menu", UIParent)
frame:SetFrameStrata("HIGH");
frame:ClearAllPoints();
frame:SetPoint("CENTER", 0, 0)
-- Size
frame:SetWidth(400);
frame:SetHeight(430);
-- Make frame movable
frame:SetMovable(true)
frame:EnableMouse(true)
frame:RegisterForDrag("RightButton")
frame:SetScript("OnDragStart", frame.StartMoving)
frame:SetScript("OnDragStop", frame.StopMovingOrSizing);
-- Backdrop
frame:SetBackdrop({
    bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
    edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
    tile = true,
    edgeSize = 16,
    tileSize = 32
});
-- Show
frame:Hide()


-- Title text
--global.utils.createTextLabel
--local elemTextTitle = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
--elemTextTitle:ClearAllPoints();
--elemTextTitle:SetPoint("TOP", frame, "TOP", 0, 20);
--elemTextTitle:SetText(global.ADDON_NAME_PRETTY .. " settings");

local checkboxYIndex = 0;
local function appendGroupCheckbox(unitLabel, settingID)

    local initialChecked = SAVED_SETTINGS_BOX_VISIBILITY[settingID];
    --print("initialChecked=" .. tostring(initialChecked));

    global.utils.createCheckbox(frame, 10, -10 - (30 * checkboxYIndex), initialChecked, unitLabel, function(self)
        local checked = self:GetChecked();
        SAVED_SETTINGS_BOX_VISIBILITY[settingID] = checked;
        global.notifyChangeBoxVisibility(settingID, checked);
        if (checked) then
            print(unitLabel .. " is now showing.")
        else
            print(unitLabel .. " is now hidden.")
        end
    end);

    checkboxYIndex = checkboxYIndex + 1;

end

-- spell type NORMAL, SHIFT, CTRL, ALT
local spellInputYIndex = 0;
local function appendSpellInput(spellDescription, settingID)

    local posX = 175;
    local posY = -20;
    local posYStep = 60;

    -- Create the text
    local elemText = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    elemText:ClearAllPoints();
    elemText:SetPoint("TOPLEFT", posX, posY - (posYStep * spellInputYIndex));
    elemText:SetText("Spell for " .. spellDescription);


    -- Create the input edit box
    local input = CreateFrame("EditBox", nil, frame)
    input:SetFontObject(ChatFontNormal);
    input:SetTextInsets(7, 0, 0, 0);
    input:SetWidth(200);
    input:SetHeight(28);

    --print("spellType=" .. spellType)
    --print("SAVED_SETTINGS_SPELLS.NORMAL=" .. SAVED_SETTINGS_SPELLS.NORMAL)

    local initialSpell = "";
    if settingID == "NORMAL" then
        initialSpell = SAVED_SETTINGS_SPELLS.NORMAL;
    elseif settingID == "SHIFT" then
        initialSpell = SAVED_SETTINGS_SPELLS.SHIFT;
    elseif settingID == "CTRL" then
        initialSpell = SAVED_SETTINGS_SPELLS.CTRL;
    elseif settingID == "ALT" then
        initialSpell = SAVED_SETTINGS_SPELLS.ALT;
    else
        print("Could not figure out spell type " .. settingID)
    end

    -- Set position
    input:ClearAllPoints();
    input:SetPoint("TOPLEFT", posX, posY - 20 - (posYStep * spellInputYIndex));
    input:SetText(initialSpell)
    -- On escape pressed
    input:SetScript("OnEscapePressed", function()
        frame:Hide()
    end)
    -- On key up
    input:SetScript("OnKeyUp", function(self)

        local newSpell = self:GetText();

        if settingID == "NORMAL" then
            SAVED_SETTINGS_SPELLS.NORMAL = newSpell;
        elseif settingID == "SHIFT" then
            SAVED_SETTINGS_SPELLS.SHIFT = newSpell;
        elseif settingID == "CTRL" then
            SAVED_SETTINGS_SPELLS.CTRL = newSpell;
        elseif settingID == "ALT" then
            SAVED_SETTINGS_SPELLS.ALT = newSpell;
        else
            print("Could not save spell type " .. settingID)
        end

        global.spellsUpdate();

    end)
    --input:HighlightText() -- select all (if to be used for copy paste)
    input:SetBackdrop({
        bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background",
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        tile = true,
        edgeSize = 16,
        tileSize = 32
    });

    spellInputYIndex = spellInputYIndex + 1;

end

local function appendCloseButton()

    global.utils.appendButton(frame, "Close", 100, "BOTTOM", 0, 10, function()
        frame:Hide();
    end)

end

local function appendResetBoxesButton()

    global.utils.appendButton(frame, "Reset box positions", 200, "TOPLEFT", 175, -340, function()
        global.notifyResetBoxesObservers();
    end)

end

local appendMiscCheckboxIndex = 0;
local function appendMiscCheckbox(label, settingID)

    local initialChecked = SAVED_SETTINGS_MISC[settingID];

    global.utils.createCheckbox(frame, 170, -260 - (30 * appendMiscCheckboxIndex), initialChecked, label, function(self)
        local checked = self:GetChecked();
        SAVED_SETTINGS_MISC[settingID] = checked;
        if (checked) then
            print(label .. " is now enabled.")
        else
            print(label .. " is now disabled.")
        end
    end);

    appendMiscCheckboxIndex = appendMiscCheckboxIndex + 1;

end

global.onInit(function()

    global.utils.createTextLabel(frame, global.ADDON_NAME_PRETTY .. " settings", "TOP", 0, 20);

    -- Append checkboxes
    appendGroupCheckbox("Player", "PLAYER")
    appendGroupCheckbox("Target", "TARGET")
    appendGroupCheckbox("Focus", "FOCUS")
    appendGroupCheckbox("Party", "PARTY")
    appendGroupCheckbox("Raid group 1", "RAID1")
    appendGroupCheckbox("Raid group 2", "RAID2")
    appendGroupCheckbox("Raid group 3", "RAID3")
    appendGroupCheckbox("Raid group 4", "RAID4")
    appendGroupCheckbox("Raid group 5", "RAID5")
    appendGroupCheckbox("Raid group 6", "RAID6")
    appendGroupCheckbox("Raid group 7", "RAID7")
    appendGroupCheckbox("Raid group 8", "RAID8")

    -- Append spell inputs
    appendSpellInput("normal click", "NORMAL");
    appendSpellInput("shift+click", "SHIFT");
    appendSpellInput("ctrl+click", "CTRL");
    appendSpellInput("alt+click", "ALT");

    -- Append misc checkbox
    appendMiscCheckbox("Snap to grid", "SNAP_TO_GRID")
    appendMiscCheckbox("Show debuffs", "SHOW_DEBUFFS")

    -- Append close button
    appendResetBoxesButton();
    appendCloseButton();

end)
