local addonName, global = ...;

local allButtons = {};

local function getClassColor(unit)

    local colorIndex = string.gsub(string.upper(UnitClass(unit)), " ", "");
    return RAID_CLASS_COLORS[colorIndex];

end

local function updateTextColor(textElement, unitLabel)

    local r, g, b = (function()

        local dR = 0.25;
        local dG = 0.25;
        local dB = 0.25;

        if (UnitIsDeadOrGhost(unitLabel))
        then
            return dR, dG, dB;
        end

        if (not UnitIsFriend("player", unitLabel))
        then
            return dR, dG, dB;
        end

        if (not UnitIsVisible(unitLabel))
        then
            return dR, dG, dB;
        end

        local currentHealth = UnitHealth(unitLabel);
        local maxHealth = UnitHealthMax(unitLabel);
        local ratio = currentHealth / maxHealth;

        if (IsSpellInRange(SAVED_SETTINGS_SPELLS.NORMAL, unitLabel) == 0)
        then
            return dR, dG, dB;
        end

        if ((ratio >= 0) and (ratio <= 1))
        then
            if (ratio < 0.5)
            then
                return 1, (ratio / 0.5), 0;
            else
                return (1 - ratio), 1, 0;
            end
        else
            return 1, 1, 1;
        end

    end)();

    textElement:SetTextColor(r, g, b);

end

local function updateTextLabel(textElement, unitLabel)

    local text = (function()

        if (not UnitExists(unitLabel))
        then
            return "?";
        end

        if (UnitIsCharmed(unitLabel))
        then
            return "?";
        end

        if (UnitIsDeadOrGhost(unitLabel))
        then
            return "X";
        end

        if (not UnitIsFriend("player", unitLabel))
        then
            return "X";
        end

        local currentHealth = UnitHealth(unitLabel);
        local maxHealth = UnitHealthMax(unitLabel);
        return tostring(floor((currentHealth / maxHealth) * 100));

    end)();

    textElement:SetText(text);

end

local function updateTextSize(textElement, unitLabel)

    local size = (function()

        local defaultFontSize = 9;

        if (not UnitExists(unitLabel))
        then
            return defaultFontSize;
        end

        if (UnitIsCharmed(unitLabel))
        then
            return defaultFontSize;
        end

        if (UnitIsDeadOrGhost(unitLabel))
        then
            return defaultFontSize;
        end

        if (not UnitIsFriend("player", unitLabel))
        then
            return defaultFontSize;
        end

        -- calculate their health ratio
        local currentHealth = UnitHealth(unitLabel);
        local maxHealth = UnitHealthMax(unitLabel);

        if (currentHealth == maxHealth) then
            return defaultFontSize;
        end

        --print("ratio1=" .. tostring((currentHealth / maxHealth)));
        --print("ratio2=" .. tostring(ratio));

        local ratio = (currentHealth / maxHealth);

        --print("ratio=" .. tostring(ratio));

        local i = 20 * (1 - ratio);
        --print("i=" .. tostring(i));
        return defaultFontSize + i;

    end)();

    --local fontSize = getUnitData(unitLabel);
    --print("textcomponent=");
    --print(textComponent);
    --print("size=" .. tostring(size));

    local path, _size, flags = textElement:GetFont();
    textElement:SetFont(path, size, flags);

end

local function showUnitTooltip(button, unitLabel)
    --print("showing unit tool tip being");
    local name = UnitName(unitLabel);

    if (name == nil) then
        name = "Unknown";
    end

    local cR = 1;
    local cG = 0.82;
    local cB = 0;

    if (UnitIsPlayer(unit))
    then
        local color = getClassColor(unitLabel);
        if (color)
        then
            cR = color.r;
            cG = color.g;
            cB = color.b;
        end
    end

    GameTooltip:SetOwner(button, "ANCHOR_BOTTOMRIGHT")
    GameTooltip:SetText(name, cR, cG, cB)
    GameTooltip:AddLine(unitLabel, 0.5, 0.5, 0.5)
    GameTooltip:Show();
    --print("showing unit tool tip END");
end

local function updateBackgroundColor(button, unitLabel)

    local r, g, b = (function()

        local cR = 0;
        local cG = 0;
        local cB = 0;

        if (not SAVED_SETTINGS_MISC.SHOW_DEBUFFS)
        then
            return cR, cG, cB;
        end

        if (UnitIsDeadOrGhost(unitLabel))
        then
            return cR, cG, cB;
        end

        if (not UnitExists(unitLabel))
        then
            return cR, cG, cB;
        end

        if (not UnitIsFriend("player", unitLabel))
        then
            return cR, cG, cB;
        end

        if (not UnitIsVisible(unitLabel))
        then
            return cR, cG, cB;
        end

        if (CheckInteractDistance(unitLabel, 4) == nil)
        then
            return cR, cG, cB;
        end

        --
        -- Check the debuffs
        --

        for i = 1, 40
        do

            local dbName, _, _, dbType = UnitDebuff(unitLabel, i, 1, "HARMFUL");

            --if (dbType ~= nil) then
            --    print(dbType);
            --end

            if (dbType)
            then

                if (dbType == "Poison")
                then
                    return 0, 0.75, 0;
                end

                if (dbType == "Magic")
                then
                    return 0, 0, 0.75;
                end

                if (dbType == "Disease")
                then
                    return 0.75, 0, 0;
                end

                if (dbType == "Curse")
                then
                    return 0.75, 0, 0.75;
                end

            end

        end

    end)();

    button:SetBackdropColor(r, g, b);

end

local function updateBorderColor(frame, unitLabel)

    local cR = 0.25;
    local cG = 0.25;
    local cB = 0.25;

    --if (UnitIsDeadOrGhost(unitLabel) ~= nil)
    --then
    --
    --    cR = 0.5;
    --    cG = 0.5;
    --    cB = 0.5;
    --
    --else
    if (UnitIsPlayer(unitLabel) and UnitClass(unitLabel) ~= nil)
    then
        local color = getClassColor(unitLabel);
        if (color)
        then
            cR = color.r;
            cG = color.g;
            cB = color.b;
        end
    end

    frame:SetBackdropBorderColor(cR, cG, cB);
    --frame:SetBackdropBorderColor( cR, cG, cB );

end

local function createUnitFrame(unitLabel, settingID)

    local frameName = "OSBCP_Easy_Healer_SecureButton_" .. unitLabel;
    local frame = CreateFrame("Button", frameName, UIParent, "SecureActionButtonTemplate");
    frame:ClearAllPoints();
    frame:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
    frame:SetWidth(50);
    frame:SetHeight(50);

    frame:SetBackdrop({
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        tile = true,
        edgeSize = 16,
        tileSize = 32,
        insets = { left = 3, right = 3, top = 3, bottom = 3 },
    });

    --frame:SetBackdropColor(0, 0, 0, 1)
    frame:SetBackdropBorderColor(1, 0, 0, 0.75);

    -- Make it draggable
    frame:SetMovable(true);
    frame:EnableMouse(true);
    frame:RegisterForDrag("RightButton")
    frame:SetScript("OnDragStart", function(self, button)
        if (not SAVED_SETTINGS_MISC.BOXES_LOCKED)
        then
            self:StartMoving();
        end
    end)
    --frame:SetScript("OnDragStart", frame.StartMoving)
    frame:SetScript("OnDragStop", function(self, button)
        if (not SAVED_SETTINGS_MISC.BOXES_LOCKED)
        then
            self:StopMovingOrSizing();
            if (SAVED_SETTINGS_MISC.SNAP_TO_GRID)
            then
                local gridStep = self:GetWidth();
                --local gridStep = self:GetWidth() + 1;
                --local gridStep = 25;
                local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint(1)
                xOfs = floor(xOfs / gridStep) * gridStep;
                yOfs = floor(yOfs / gridStep) * gridStep;
                self:ClearAllPoints();
                self:SetPoint(point, relativeTo, relativePoint, xOfs, yOfs)
            end
        end
    end)
    --frame:SetScript("OnDragStop", frame.StopMovingOrSizing);

    -- Create the text
    frame.text = frame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
    --frame.text:SetFont(GameFontNormal, 13, "OUTLINE")
    frame.text:SetPoint("CENTER", 0, 0)
    frame.text:SetText("#text#");

    -- On click events
    frame:SetAttribute("*type1", "spell");
    --frame:SetAttribute("spell1", SAVED_SETTINGS_SPELLS.NORMAL);
    --frame:SetAttribute("shift-spell1", SAVED_SETTINGS_SPELLS.SHIFT);
    --frame:SetAttribute("ctrl-spell1", SAVED_SETTINGS_SPELLS.CTRL);
    --frame:SetAttribute("alt-spell1", SAVED_SETTINGS_SPELLS.ALT);
    frame:SetAttribute("unit", unitLabel);

    -- Tooltip
    frame:SetScript("OnEnter", function(self, button)
        --print("OnEnter")
        showUnitTooltip(self, unitLabel)
    end)
    frame:SetScript("OnLeave", function(self, button)
        --print("OnLeave")
        GameTooltip:Hide();
    end)
    --frame:SetScript("OnClick", function(self, button)
    --print("OnLeave")
    --GameTooltip:Hide();
    --end)

    -- Update
    frame:SetScript("OnUpdate", function(self, button)
        --print(SAVED_SETTINGS_BOX_VISIBILITY[settingID]);
        --print(settingID);
        local visible = SAVED_SETTINGS_BOX_VISIBILITY[settingID];
        --print("visible=" .. tostring(visible));
        if (visible) then

            updateBorderColor(self, unitLabel);
            updateBackgroundColor(self, unitLabel);
            updateTextColor(self.text, unitLabel);
            updateTextLabel(self.text, unitLabel);
            updateTextSize(self.text, unitLabel);

            --self:Show();
        else
            self:Hide();
        end
    end)

    global.onResetBox(function()
        frame:ClearAllPoints();
        frame:SetPoint("CENTER", 0, 0);
    end);

    global.onChangeBoxVisibility(function(sID, visible)
        --print("onChangeBoxVisibility " .. sID .. tostring(visible))
        if (settingID ~= sID)
        then
            return ;
        end
        if (visible)
        then
            frame:Show();
        else
            frame:Hide();
        end
    end);

    table.insert(allButtons, frame);

    return frame;

end

global.onSpellsUpdate(function()

    --print("** on spells updated ");

    for i, button in ipairs(allButtons) do
        --button:SetAttribute("*type1", "spell");
        button:SetAttribute("spell1", SAVED_SETTINGS_SPELLS.NORMAL);
        button:SetAttribute("shift-spell1", SAVED_SETTINGS_SPELLS.SHIFT);
        button:SetAttribute("ctrl-spell1", SAVED_SETTINGS_SPELLS.CTRL);
        button:SetAttribute("alt-spell1", SAVED_SETTINGS_SPELLS.ALT);
        --button:SetAttribute("unit", unit);
    end

end);

-- Init
(function()

    createUnitFrame("player", "PLAYER");
    createUnitFrame("target", "TARGET");
    createUnitFrame("focus", "FOCUS");

    -- Party
    for i = 1, 4, 1
    do
        createUnitFrame("party" .. i, "PARTY");
    end

    for i = 1, 8, 1
    do
        createUnitFrame("raid" .. i, "RAID" .. i);
    end

end)();
