local addonName, global = ...;

global.ADDON_NAME_PRETTY = "OSBCP Easy Healer";
global.VERSION = "1.0.0";

local initObservers = {}
global.onInit = function(observer)
    table.insert(initObservers, observer);
end

local spellsUpdatedObservers = {}
global.onSpellsUpdate = function(observer)
    table.insert(spellsUpdatedObservers, observer);
end
global.spellsUpdate = function()

    for i, observer in ipairs(spellsUpdatedObservers) do
        observer();
    end

end

local resetBoxesObservers = {}
global.notifyResetBoxesObservers = function()
    for i, observer in ipairs(resetBoxesObservers) do
        observer();
    end
end
global.onResetBox = function(observer)
    table.insert(resetBoxesObservers, observer);
end

local changeBoxVisibilityObservers = {}
global.notifyChangeBoxVisibility = function(settingID, visible)
    for i, observer in ipairs(changeBoxVisibilityObservers) do
        observer(settingID, visible);
    end
end
global.onChangeBoxVisibility = function(observer)
    table.insert(changeBoxVisibilityObservers, observer);
end

global.init = function()

    -- Default values
    if (SAVED_SETTINGS_MISC == nil) then
        --print("creating defaults for saved settings")
        SAVED_SETTINGS_MISC = {
            BOXES_LOCKED = false,
            SHOW_DEBUFFS = true,
            SNAP_TO_GRID = true,
        }
    end
    if (SAVED_SETTINGS_SPELLS == nil) then
        --print("creating defaults for saved settings SPELLS")
        SAVED_SETTINGS_SPELLS = {
            NORMAL = "Flash of Light",
            SHIFT = "Consecration",
            CTRL = "Holy Light",
            ALT = "Seal of Light"
        }
    end
    if (SAVED_SETTINGS_BOX_VISIBILITY == nil) then
        --print("creating defaults for saved settings SAVED_SETTINGS_BOX_VISIBILITY")
        SAVED_SETTINGS_BOX_VISIBILITY = {
            PLAYER = true,
            TARGET = false,
            FOCUS = false,
            PARTY = false,
            RAID1 = false,
            RAID2 = false,
            RAID3 = false,
            RAID4 = false,
            RAID5 = false,
            RAID6 = false,
            RAID7 = false,
            RAID8 = false
        }
    end

    --print("*** global init **")

    for i, observer in ipairs(initObservers) do
        observer();
    end

    for i, observer in ipairs(spellsUpdatedObservers) do
        observer();
    end

end
